/**
 * CRXProxy.cpp
 */

#include "CRXProxy.h"

#include <string.h>

CRXFilter	CRXProxy::mFilter;

CRXProxy::CRXProxy (const int aSocket)
	: mServerTimeout (0)
{
	/*----------------------------------------------------------------*/
	if (aSocket < 0)
		return ;

	mClient = aSocket;
	/*----------------------------------------------------------------*/
}

CRXProxy::~CRXProxy (void)
{
	/*----------------------------------------------------------------*/
	Close ();
	/*----------------------------------------------------------------*/
}

void
CRXProxy::SetServerTimeout (const int aTimeout)
{
	/*----------------------------------------------------------------*/
	mServerTimeout = aTimeout;
	/*----------------------------------------------------------------*/
}

void
CRXProxy::Close (void)
{
	/*----------------------------------------------------------------*/
	mServer.Close ();
	mClient.Close ();
	/*----------------------------------------------------------------*/
}

int
CRXProxy::Forward (void)
{
	int		aResult = 0;

	/*----------------------------------------------------------------*/
	aResult = ReceiveRequest ();
	if (aResult < 0)
	{
		aResult = ERROR_PROXY_FAILED_TO_RECEIVE_REQUEST;
		CRX_ERROR_SET (aResult, "Failed to receive from client.");
	}

	aResult = SendRequest ();
	if (aResult < 0)
	{
		aResult = ERROR_PROXY_FAILED_TO_SEND_REQUEST;
		CRX_ERROR_SET (aResult, "Failed to send to server.");
		return aResult;
	}

	aResult = ReceiveResponse ();
	if (aResult < 0)
	{
		aResult = ERROR_PROXY_FAILED_TO_RECEIVE_RESPONSE;
		CRX_ERROR_SET (aResult, "Failed to receive from server.");
	}

	aResult = SendResponse ();
	if (aResult < 0)
	{
		aResult = ERROR_PROXY_FAILED_TO_SEND_RESPONSE;
		CRX_ERROR_SET (aResult, "Failed to send to client.");
		return aResult;
	}
	/*----------------------------------------------------------------*/

	return aResult;
}

int
CRXProxy::ReceiveRequest (void)
{
	int			aResult = 0;
	char		aBuffer;

	std::string	aHttpHeader;
	/*----------------------------------------------------------------*/
	for (;;)
	{
		aResult = mClient.Receive (&aBuffer, 1);
		if (aResult < 0)
		{
			aResult = ERROR_PROXY_FAILED_TO_RECEIVE_REQUEST;
			CRX_ADD_SUBCLASS_ERROR (mClient);
			CRX_ERROR_SET (aResult, "Failed to receive from client.");
			return aResult;
		}
		aHttpHeader += aBuffer;
		if (aHttpHeader.rfind (CRLF2) != std::string::npos)
			break;
	}

	mHttpRequest.SetHeader (aHttpHeader.c_str ());
	/*----------------------------------------------------------------*/

	return aResult < 0 ? aResult : 0;
}

int
CRXProxy::SendRequest (void)
{
	int		aResult	= 0;
	int		aSize	= mHttpRequest.GetHeader ().length ();

	struct sockaddr_in	aSockName;
	socklen_t			aSockLen = sizeof (aSockName);

	/*----------------------------------------------------------------*/
	if (!mServer)
	{
		/* connect */
		aResult = mServer.Connect (mHttpRequest.GetHost (), mHttpRequest.GetPort (), mServerTimeout);
		if (aResult < 0)
		{
			aResult = ERROR_PROXY_FAILED_TO_CONNECT_TO_SERVER;
			CRX_ADD_SUBCLASS_ERROR (mServer);
			CRX_ERROR_SET (aResult, "Failed to connect to server <%s>.", mHttpRequest.GetURL ().c_str ());
			return aResult;
		}
	}

	aResult = mServer.Send (mHttpRequest.GetHeader ().c_str (), aSize);
	if (aResult != aSize)
	{
		CRX_ADD_SUBCLASS_ERROR (mServer);
		CRX_ERROR_SET (aResult, "Failed to send to server <%s>.", mHttpRequest.GetURL ().c_str ());
	}
	/*----------------------------------------------------------------*/

	return aResult < 0 ? aResult : 0;
}

int
CRXProxy::ReceiveResponse (void)
{
	int					aResult			= 0;

	const unsigned int	aBufferSize		= 1024 * 64;
	char				aBuffer[aBufferSize + 1] = {0x00, };

	/*----------------------------------------------------------------*/
	for (;;)
	{
		memset (aBuffer, 0x00, sizeof (aBuffer));

		aResult = mServer.Receive (aBuffer, aBufferSize);
		if (aResult < 0)
		{
			aResult = ERROR_PROXY_FAILED_TO_RECEIVE_REQUEST;
			CRX_ADD_SUBCLASS_ERROR (mServer);
			CRX_ERROR_SET (aResult, "Failed to receive from server.");
			return aResult;
		}

		aResult = mHttpResponse.SetResponseAll (aBuffer, aResult);
		if (aResult < 0)
		{
			aResult = ERROR_PROXY_FAILED_TO_SET_RESPONSE;
			CRX_ERROR_SET (aResult, "Failed to set response.");
			return aResult;
		}
	}
	/*----------------------------------------------------------------*/

	return aResult < 0 ? aResult : 0;
}

int
CRXProxy::SendResponse (void)
{
	int		aResult	= 0;
	int		aSize	= 0;

	/*----------------------------------------------------------------*/
	aSize = mHttpResponse.GetHeader ().length ();;
	aResult = mClient.Send (mHttpResponse.GetHeader ().c_str (), aSize);
	if (aResult != aSize)
	{
		CRX_ADD_SUBCLASS_ERROR (mClient);
		CRX_ERROR_SET (aResult, "Failed to send to client.");
		return aResult;
	}

	aSize = mHttpResponse.GetContentLength ();
	aResult = mClient.Send (mHttpResponse.GetContentBody (), aSize);
	if (aResult != aSize)
	{
		CRX_ADD_SUBCLASS_ERROR (mClient);
		CRX_ERROR_SET (aResult, "Failed to send to client.");
	}
	/*----------------------------------------------------------------*/

	return aResult < 0 ? aResult : 0;
}

CRXFilter &
CRXProxy::GetFilter (void) const
{
	/*----------------------------------------------------------------*/
	/*----------------------------------------------------------------*/

	return mFilter;
}

CRXHttpRequest &
CRXProxy::GetHttpRequest (void)
{
	/*----------------------------------------------------------------*/
	/*----------------------------------------------------------------*/

	return mHttpRequest;
}

CRXHttpResponse &
CRXProxy::GetHttpResponse (void)
{
	/*----------------------------------------------------------------*/
	/*----------------------------------------------------------------*/

	return mHttpResponse;
}